# Generated by Django 5.1.7 on 2025-04-10 10:10

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ResilientLogEntry",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                ("is_sent", models.BooleanField(default=False, verbose_name="is sent")),
                ("level", models.IntegerField(default=0, verbose_name="level")),
                ("message", models.JSONField(verbose_name="message")),
                ("context", models.JSONField(null=True, verbose_name="context")),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="created at"),
                ),
            ],
            options={
                "verbose_name_plural": "resilient log entries",
            },
        ),
    ]



import logging

from django.core.management.base import BaseCommand

from resilient_logger.resilient_logger import ResilientLogger
from resilient_logger.utils import get_resilient_logger_config

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = (
        "Clear django-resilient-logger entries which are already submitted,"
        "only clear if settings.CLEAR_SENT_ENTRIES is set to True (default: False)"
    )

    def __init__(self):
        super().__init__()
        settings = get_resilient_logger_config()
        self.should_clear = settings["clear_sent_entries"] or False
        self.resilient_logger = ResilientLogger.create()

    def add_arguments(self, parser):
        parser.add_argument(
            "--days-to-keep",
            action="store",
            dest="days_to_keep",
            type=int,
            default=30,
            help="Days to keep the old values stored",
        )

    def handle(self, *args, **options):
        days_to_keep = options.get("days_to_keep", 30)

        if not self.should_clear:
            logger.info("clear_sent_entries is disabled in config")
            return

        logger.info("Begin clear_sent_entries job")
        result = self.resilient_logger.clear_sent_entries(days_to_keep)
        logger.info("Finished clear_sent_entries job done", extra={"result": result})



from .abstract_log_target import AbstractLogTarget as AbstractLogTarget
from .elasticsearch_log_target import ElasticsearchLogTarget as ElasticsearchLogTarget
from .proxy_log_target import ProxyLogTarget as ProxyLogTarget



import logging
from contextlib import nullcontext as does_not_raise

import pytest
from django.test import override_settings

from resilient_logger.errors import MissingContextError
from resilient_logger.handlers import ResilientLogHandler
from tests.testdata.testconfig import (
    VALID_CONFIG_ALL_FIELDS,
)

without_required_fields_with_extras = ([], {"foo": "bar"}, does_not_raise())
without_required_fields_without_extras = ([], {}, does_not_raise())
with_required_fields_with_extras = (["foo"], {"foo": "bar"}, does_not_raise())
with_required_fields_without_extras = (
    ["foo"],
    {},
    pytest.raises(MissingContextError),
)


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_without_required_fields_with_extras():
    required_fields, extra, expectation = without_required_fields_with_extras

    logger = logging.Logger(__name__)
    logger.addHandler(
        ResilientLogHandler(logging.INFO, required_fields=required_fields)
    )

    with expectation:
        logger.info("Hello World!", extra=extra)


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_without_required_fields_without_extras():
    required_fields, extra, expectation = without_required_fields_without_extras

    logger = logging.Logger(__name__)
    logger.addHandler(
        ResilientLogHandler(logging.INFO, required_fields=required_fields)
    )

    with expectation:
        logger.info("Hello World!", extra=extra)


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_with_required_fields_with_extras():
    required_fields, extra, expectation = with_required_fields_with_extras

    logger = logging.Logger(__name__)
    logger.addHandler(
        ResilientLogHandler(logging.INFO, required_fields=required_fields)
    )

    with expectation:
        logger.info("Hello World!", extra=extra)


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_with_required_fields_without_extras():
    required_fields, extra, expectation = with_required_fields_without_extras

    logger = logging.Logger(__name__)
    logger.addHandler(
        ResilientLogHandler(logging.INFO, required_fields=required_fields)
    )

    with expectation:
        logger.info("Hello World!", extra=extra)



import datetime
from collections.abc import Iterable, Iterator
from dataclasses import dataclass
from typing import Any, TypeVar

from django.db import transaction
from django.utils import timezone

from resilient_logger.models import ResilientLogEntry
from resilient_logger.sources import AbstractLogSource
from resilient_logger.sources.abstract_log_source import AuditLogDocument
from resilient_logger.utils import get_resilient_logger_config, value_as_dict

TResilientLogSource = TypeVar("TResilientLogSource", bound="ResilientLogSource")


@dataclass
class ResilientLogEntryData:
    level: int
    message: Any
    context: dict


@dataclass
class StructuredResilientLogEntryData:
    message: Any
    level: int = 0
    operation: str = "MANUAL"
    actor: dict | None = None
    target: dict | None = None
    extra: dict | None = None


class ResilientLogSource(AbstractLogSource):
    def __init__(self, log: ResilientLogEntry):
        self.log = log

    @classmethod
    def create(
        cls: type[TResilientLogSource], *, level: int, message: Any, context: dict
    ) -> TResilientLogSource:
        entry = ResilientLogEntry.objects.create(
            level=level,
            message=message,
            context=context,
        )

        return cls(entry)

    @classmethod
    def bulk_create(
        cls: type[TResilientLogSource], objs: Iterable[ResilientLogEntryData]
    ) -> Iterable[TResilientLogSource]:
        entries = ResilientLogEntry.objects.bulk_create(
            ResilientLogEntry(level=obj.level, message=obj.message, context=obj.context)
            for obj in objs
        )

        return [cls(entry) for entry in entries]

    @classmethod
    def create_structured(
        cls: type[TResilientLogSource],
        *,
        message: Any,
        level: int = 0,
        operation: str = "MANUAL",
        actor: dict | None = None,
        target: dict | None = None,
        extra: dict | None = None,
    ) -> TResilientLogSource:
        return cls.create(
            level=level,
            message=message,
            context={
                **(extra or {}),
                "actor": actor or {},
                "operation": operation,
                "target": target or {},
            },
        )

    @classmethod
    def bulk_create_structured(
        cls: type[TResilientLogSource],
        objs: list[StructuredResilientLogEntryData],
    ) -> Iterable[TResilientLogSource]:
        prepared_objs = [
            ResilientLogEntryData(
                level=obj.level,
                message=obj.message,
                context={
                    **(obj.extra or {}),
                    "actor": obj.actor or {},
                    "operation": obj.operation,
                    "target": obj.target or {},
                },
            )
            for obj in objs
        ]

        return cls.bulk_create(objs=prepared_objs)

    def get_id(self) -> str | int:
        return self.log.id

    def get_document(self) -> AuditLogDocument:
        config = get_resilient_logger_config()
        context = (self.log.context or {}).copy()
        actor = context.pop("actor", "unknown")
        operation = context.pop("operation", "MANUAL")
        target = context.pop("target", "unknown")
        iso_date = (
            self.log.created_at.astimezone(datetime.timezone.utc)
            .isoformat(timespec="milliseconds")
            .replace("+00:00", "Z")
        )

        extra = {
            **context,
            "source_pk": self.get_id(),
        }

        return {
            "@timestamp": iso_date,
            "audit_event": {
                "actor": value_as_dict(actor),
                "date_time": iso_date,
                "operation": operation,
                "origin": config["origin"],
                "target": value_as_dict(target),
                "environment": config["environment"],
                "message": self.log.message,
                "level": self.log.level,
                "extra": extra,
            },
        }

    def is_sent(self) -> bool:
        return self.log.is_sent

    def mark_sent(self) -> None:
        self.log.is_sent = True
        self.log.save(update_fields=["is_sent"])

    @classmethod
    @transaction.atomic
    def get_unsent_entries(cls, chunk_size: int) -> Iterator["ResilientLogSource"]:
        entries = (
            ResilientLogEntry.objects.filter(is_sent=False)
            .order_by("created_at")
            .iterator(chunk_size=chunk_size)
        )

        for entry in entries:
            yield cls(entry)

    @classmethod
    @transaction.atomic
    def clear_sent_entries(cls, days_to_keep: int = 30) -> list[str]:
        entries = ResilientLogEntry.objects.filter(
            is_sent=True,
            created_at__lte=(timezone.now() - datetime.timedelta(days=days_to_keep)),
        ).select_for_update()

        deleted_ids = list(entries.values_list("id", flat=True))
        entries.delete()

        return [str(deleted_id) for deleted_id in deleted_ids]



import pytest

from resilient_logger.models import ResilientLogEntry
from resilient_logger.sources import ResilientLogSource
from resilient_logger.sources.resilient_log_source import (
    StructuredResilientLogEntryData,
)


@pytest.mark.django_db
def test_bulk_create_resilient_log_entries(django_assert_max_num_queries):
    with django_assert_max_num_queries(1):
        ResilientLogSource.bulk_create_structured(
            [
                StructuredResilientLogEntryData(
                    message="Hello world", extra={"index": idx}
                )
                for idx in range(10)
            ]
        )

    for idx in range(10):
        ResilientLogEntry.objects.get(message="Hello world", context__index=idx)



import logging

from resilient_logger.utils import assert_required_extras, get_log_record_extra

logger = logging.getLogger(__name__)


class ResilientLogHandler(logging.Handler):
    def __init__(
        self,
        level: int = logging.NOTSET,
        required_fields: list[str] | None = None,
    ):
        super().__init__(level)
        self.required_fields = required_fields or []

    def emit(self, record: logging.LogRecord):
        """
        ResilientLoggerSource rely on Django's DB models and cannot be imported during
        init since Django app registry is not ready by then.

        To work around this, import it here when the logger is used first time.
        """
        from resilient_logger.sources import ResilientLogSource

        extra = get_log_record_extra(record)
        assert_required_extras(extra, self.required_fields)

        return ResilientLogSource.create(
            level=record.levelno, message=record.getMessage(), context=extra
        )



import logging

import pytest
from django.core.management import call_command
from django.test import override_settings

from resilient_logger.models import ResilientLogEntry
from resilient_logger.sources import ResilientLogSource
from resilient_logger.utils import get_resilient_logger_config
from tests.testdata.testconfig import (
    VALID_CONFIG_ALL_FIELDS,
    VALID_CONFIG_MISSING_OPTIONAL,
)

logger = logging.getLogger(__name__)


def extract_result(record: logging.LogRecord):
    # Can change, but for now the command stored the result in extra["result"]
    return record.__dict__["result"]


def create_resilient_log_entries(count: int, mark_sent: bool):
    for idx in range(count):
        entry = ResilientLogSource.create_structured(
            message="Hello world", extra={"index": idx}
        )

        if mark_sent:
            entry.mark_sent()


@pytest.fixture(autouse=True)
def clear_config_cache():
    get_resilient_logger_config.cache_clear()
    yield


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_submit_unsent_entries(caplog: pytest.LogCaptureFixture):
    logger_name = "resilient_logger.management.commands.submit_unsent_entries"
    num_log_entries = 10
    create_resilient_log_entries(num_log_entries, False)

    with caplog.at_level(logging.INFO, logger=logger_name):
        call_command("submit_unsent_entries")
        result = extract_result(caplog.records[1])
        assert len(result) == num_log_entries
        caplog.clear()

    with caplog.at_level(logging.INFO, logger=logger_name):
        call_command("submit_unsent_entries")
        result = extract_result(caplog.records[1])
        assert len(result) == 0
        caplog.clear()


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_clear_sent_entries(caplog: pytest.LogCaptureFixture):
    logger_name = "resilient_logger.management.commands.clear_sent_entries"
    num_log_entries = 10
    create_resilient_log_entries(num_log_entries, True)

    with caplog.at_level(logging.INFO, logger=logger_name):
        call_command("clear_sent_entries", days_to_keep=0)
        result = extract_result(caplog.records[1])
        assert len(result) == num_log_entries
        caplog.clear()

    with caplog.at_level(logging.INFO, logger=logger_name):
        call_command("clear_sent_entries", days_to_keep=0)
        result = extract_result(caplog.records[1])
        assert len(result) == 0
        caplog.clear()


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_MISSING_OPTIONAL)
def test_submit_unsent_entries_disabled(caplog: pytest.LogCaptureFixture):
    logger_name = "resilient_logger.management.commands.submit_unsent_entries"
    num_log_entries = 5

    create_resilient_log_entries(num_log_entries, False)

    with caplog.at_level(logging.INFO, logger=logger_name):
        call_command("submit_unsent_entries")

        assert "submit_unsent_entries is disabled in config" in caplog.text
        assert "Begin submit_unsent_entries job" not in caplog.text

        assert (
            ResilientLogEntry.objects.filter(is_sent=False).count() == num_log_entries
        )


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_MISSING_OPTIONAL)
def test_clear_sent_entries_disabled(caplog: pytest.LogCaptureFixture):
    logger_name = "resilient_logger.management.commands.clear_sent_entries"
    num_log_entries = 5

    create_resilient_log_entries(num_log_entries, True)

    with caplog.at_level(logging.INFO, logger=logger_name):
        call_command("clear_sent_entries", days_to_keep=0)

        assert "clear_sent_entries is disabled in config" in caplog.text
        assert "Begin clear_sent_entries job" not in caplog.text
        assert ResilientLogEntry.objects.filter(is_sent=True).count() == num_log_entries



import logging
from urllib.parse import urlparse

from elasticsearch8 import ConflictError, Elasticsearch

from resilient_logger.sources import AbstractLogSource
from resilient_logger.targets import AbstractLogTarget
from resilient_logger.utils import content_hash

# Constants
ES_STATUS_CREATED = "created"

logger = logging.getLogger(__name__)


class ElasticsearchLogTarget(AbstractLogTarget):
    """
    Log target that sends entries to Elasticsearch.

    The constructor requires connection details, including `es_username`, `es_password`,
    and `es_index`.

    You can specify the connection using either a full `es_url` or separate fields:
    `es_host`, `es_port`, and `es_scheme`.

    If `es_url` is provided but lacks a scheme or port, the values from `es_scheme` and
    `es_port` will be used.

    Defaults:
    - `es_scheme`: https
    - `es_port`: 9200
    """

    def __init__(
        self,
        *,
        es_username: str,
        es_password: str,
        es_index: str,
        es_url: str | None = None,
        es_host: str | None = None,
        es_port: int | None = 9200,
        es_scheme: str | None = "https",
        required: bool = True,
    ) -> None:
        super().__init__(required)

        if not es_url:
            scheme = es_scheme
            host = es_host
            port = es_port
        else:
            if "://" not in es_url:
                es_url = f"{es_scheme}://{es_url}"

            parsed = urlparse(es_url)
            scheme: str | None = parsed.scheme
            host: str | None = parsed.hostname
            port: int | None = parsed.port or es_port

        self._index = es_index
        self._client = Elasticsearch(
            [{"host": host, "port": port, "scheme": scheme}],
            basic_auth=(es_username, es_password),
        )

    def submit(self, entry: AbstractLogSource) -> bool:
        document = entry.get_document()
        hash = content_hash(document)

        try:
            response = self._client.index(
                index=self._index,
                id=hash,
                document=document,
                op_type="create",
            )

            logger.info(f"Sending status: {response}")
            result = response["result"]

            if result == ES_STATUS_CREATED:
                return True

        except ConflictError:
            """
            The document key used to store log entry is the hash of the contents.
            If we receive conflict error, it means that the given entry is already
            sent to the Elasticsearch.
            """
            logger.warning(
                f"""Skipping the document with key {hash}, it's already submitted.""",
                extra=document,
            )

            return True
        except Exception:
            """
            Unknown exception, log it and keep going to avoid transaction rollbacks.
            """
            logger.exception(f"Entry with key {hash} failed.")

        return False



from base64 import b64encode

import pytest

from resilient_logger.targets import ElasticsearchLogTarget

scheme = "https"
host = "host"
port = 1234

username = "user"
password = "password"
index = "index"

credentials_string = f"{username}:{password}"
credentials_bytes = credentials_string.encode("utf-8")
expected_authorization = f"Basic {b64encode(credentials_bytes).decode('utf-8')}"


@pytest.mark.django_db
def test_create_url_complete():
    target = ElasticsearchLogTarget(
        es_url=f"{scheme}://{host}:{port}",
        es_username=username,
        es_password=password,
        es_index=index,
    )

    client = target._client
    node = client.transport.node_pool.get()
    authorization = client._headers.get("authorization")

    assert node.scheme == scheme
    assert node.host == host
    assert node.port == port
    assert target._index == index
    assert authorization == expected_authorization


@pytest.mark.django_db
def test_create_url_without_scheme():
    target = ElasticsearchLogTarget(
        es_url=f"{host}:{port}",
        es_username=username,
        es_password=password,
        es_index=index,
    )

    client = target._client
    node = client.transport.node_pool.get()
    authorization = client._headers.get("authorization")

    assert node.scheme == scheme
    assert node.host == host
    assert node.port == port
    assert target._index == index
    assert authorization == expected_authorization


@pytest.mark.django_db
def test_create_parts_with_scheme_host_port():
    target = ElasticsearchLogTarget(
        es_scheme=scheme,
        es_host=host,
        es_port=port,
        es_username=username,
        es_password=password,
        es_index=index,
    )

    client = target._client
    node = client.transport.node_pool.get()
    authorization = client._headers.get("authorization")

    assert node.scheme == scheme
    assert node.host == host
    assert node.port == port
    assert target._index == index
    assert authorization == expected_authorization


@pytest.mark.django_db
def test_create_parts_without_scheme():
    target = ElasticsearchLogTarget(
        es_host=host,
        es_port=port,
        es_username=username,
        es_password=password,
        es_index=index,
    )

    client = target._client
    node = client.transport.node_pool.get()
    authorization = client._headers.get("authorization")

    assert node.scheme == scheme
    assert node.host == host
    assert node.port == port
    assert target._index == index
    assert authorization == expected_authorization



# Generated by Django 4.2.25 on 2025-10-29 07:07

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resilient_logger", "0003_created_at_sorting_and_index"),
    ]

    operations = [
        migrations.AlterField(
            model_name="resilientlogentry",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



import datetime

import pytest
from django.test import override_settings

from resilient_logger.utils import (
    get_resilient_logger_config,
    unavailable_class,
    value_as_dict,
)
from tests.testdata.testconfig import (
    INVALID_CONFIG_EMPTY_ENVIRONMENT,
    INVALID_CONFIG_EMPTY_ORIGIN,
    INVALID_CONFIG_EMPTY_SOURCES,
    INVALID_CONFIG_EMPTY_TARGETS,
    INVALID_CONFIG_MISSING_ENVIRONMENT,
    INVALID_CONFIG_MISSING_ORIGIN,
    INVALID_CONFIG_MISSING_SOURCES,
    INVALID_CONFIG_MISSING_TARGETS,
    VALID_CONFIG_ALL_FIELDS,
    VALID_CONFIG_MISSING_OPTIONAL,
)


@pytest.fixture(autouse=True)
def setup():
    get_resilient_logger_config.cache_clear()


@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_valid_config_all_fields():
    config = get_resilient_logger_config()
    assert config == VALID_CONFIG_ALL_FIELDS


@override_settings(RESILIENT_LOGGER=VALID_CONFIG_MISSING_OPTIONAL)
def test_valid_config_missing_optional():
    config = get_resilient_logger_config()
    assert config["batch_limit"] is not None
    assert config["chunk_size"] is not None
    assert config["submit_unsent_entries"] is not None
    assert config["clear_sent_entries"] is not None


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_MISSING_TARGETS)
def test_invalid_config_missing_targets():
    with pytest.raises(RuntimeError, match="missing required key: 'targets'"):
        get_resilient_logger_config()


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_EMPTY_TARGETS)
def test_invalid_config_empty_targets():
    with pytest.raises(
        RuntimeError, match=r"RESILIENT_LOGGER\['targets'\] failed validation"
    ):
        get_resilient_logger_config()


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_MISSING_SOURCES)
def test_invalid_config_missing_sources():
    with pytest.raises(RuntimeError, match="missing required key: 'sources'"):
        get_resilient_logger_config()


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_EMPTY_SOURCES)
def test_invalid_config_empty_sources():
    with pytest.raises(
        RuntimeError, match=r"RESILIENT_LOGGER\['sources'\] failed validation"
    ):
        get_resilient_logger_config()


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_MISSING_ENVIRONMENT)
def test_invalid_config_missing_environment():
    with pytest.raises(RuntimeError, match="missing required key: 'environment'"):
        get_resilient_logger_config()


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_EMPTY_ENVIRONMENT)
def test_invalid_config_empty_environment():
    with pytest.raises(
        RuntimeError, match=r"RESILIENT_LOGGER\['environment'\] failed validation"
    ):
        get_resilient_logger_config()


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_MISSING_ORIGIN)
def test_invalid_config_missing_origin():
    with pytest.raises(RuntimeError, match="missing required key: 'origin'"):
        get_resilient_logger_config()


@override_settings(RESILIENT_LOGGER=INVALID_CONFIG_EMPTY_ORIGIN)
def test_invalid_config_empty_origin():
    with pytest.raises(
        RuntimeError, match=r"RESILIENT_LOGGER\['origin'\] failed validation"
    ):
        get_resilient_logger_config()


def test_unavailable_class():
    with pytest.raises(
        ImportError,
        match="ClassName requires the optional dependencies: 'library-name'.",
    ):
        placeholder_class = unavailable_class("ClassName", ["library-name"])
        placeholder_class()


def test_value_as_dict():
    as_str = "hello"
    as_dict = {"value": as_str}
    invalid = datetime.datetime(2025, 10, 10)

    assert value_as_dict(as_str) == as_dict
    assert value_as_dict(as_dict) == as_dict

    with pytest.raises(TypeError) as ex:
        value_as_dict(invalid)

    assert ex.match("Expected 'str | dict', got 'datetime'")



import logging

from resilient_logger.sources import AbstractLogSource
from resilient_logger.targets import AbstractLogTarget


class ProxyLogTarget(AbstractLogTarget):
    """
    Logger target that sends the resilient log entries to another logger.
    """

    def __init__(self, name: str = __name__, required: bool = False) -> None:
        super().__init__(required)
        self._logger = logging.getLogger(name)

    def submit(self, entry: AbstractLogSource) -> bool:
        document = entry.get_document()
        audit_event = document["audit_event"]
        actor = audit_event.get("actor", "unknown")
        operation = audit_event.get("operation", "MANUAL")
        target = audit_event.get("target", "unknown")
        message: str = audit_event.pop("message")
        level = audit_event.get("level", logging.INFO)
        extra = audit_event.get("extra", {})

        context = {**extra, "actor": actor, "operation": operation, "target": target}
        self._logger.log(level=level, msg=message, extra=context)
        return True



import json
import logging

from django.contrib import admin
from django.utils.html import escape
from django.utils.safestring import mark_safe

from resilient_logger.models import ResilientLogEntry

logger = logging.getLogger(__name__)


@admin.register(ResilientLogEntry)
class ResilientLogEntryAdmin(admin.ModelAdmin):
    exclude = ("message", "context")
    readonly_fields = (
        "id",
        "is_sent",
        "level",
        "created_at",
        "message_prettified",
        "context_prettified",
    )
    list_display = ("id", "__str__", "created_at", "is_sent")
    list_filter = ("created_at", "is_sent")

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    @admin.display(description="message")
    def message_prettified(self, instance):
        """Format the message to be a bit a more user-friendly."""
        message = json.dumps(instance.message, indent=2, sort_keys=True)
        content = f"<pre>{escape(message)}</pre>"
        return mark_safe(content)

    @admin.display(description="context")
    def context_prettified(self, instance):
        """Format the context to be a bit a more user-friendly."""
        context = json.dumps(instance.context, indent=2, sort_keys=True)
        content = f"<pre>{escape(context)}</pre>"
        return mark_safe(content)



import importlib
from unittest.mock import patch

import pytest
from auditlog.models import LogEntry
from django.test import override_settings

from resilient_logger.sources import DjangoAuditLogSource
from tests.models import DummyModel
from tests.testdata.testconfig import VALID_CONFIG_ALL_FIELDS


def create_objects(count: int) -> list[DummyModel]:
    results: list[DummyModel] = []

    for i in range(count):
        results.append(DummyModel.objects.create(message=str(i)))

    return results


def object_to_auditlog_source(model: DummyModel) -> DjangoAuditLogSource:
    entry = LogEntry.objects.get(object_pk=model.id)
    return DjangoAuditLogSource(entry)


@pytest.mark.django_db
def test_mark_sent():
    [object] = create_objects(1)

    source = object_to_auditlog_source(object)
    assert not source.is_sent()

    source.mark_sent()
    assert source.is_sent()


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_get_unsent_entries():
    num_objects = 3
    objects = create_objects(num_objects)

    all_log_entries = LogEntry.objects.filter()
    assert len(all_log_entries) == num_objects

    for log_entry in all_log_entries:
        assert not log_entry.additional_data

    actual_entries = [object_to_auditlog_source(obj) for obj in objects]
    unsent_entries = list(DjangoAuditLogSource.get_unsent_entries(500))

    assert len(actual_entries) == len(unsent_entries)

    for i in range(num_objects):
        assert actual_entries[i].get_id() == unsent_entries[i].get_id()
        assert actual_entries[i].get_document() == unsent_entries[i].get_document()
        actual_entries[i].mark_sent()

    unsent_entries = list(DjangoAuditLogSource.get_unsent_entries(500))
    assert len(unsent_entries) == 0

    for log_entry in all_log_entries:
        log_entry.refresh_from_db()
        assert log_entry.additional_data["is_sent"]


@pytest.mark.django_db
@override_settings(RESILIENT_LOGGER=VALID_CONFIG_ALL_FIELDS)
def test_clear_sent_entries():
    num_objects = 3
    objects = create_objects(num_objects)
    actual_entries = [object_to_auditlog_source(obj) for obj in objects]

    for actual_entry in actual_entries:
        actual_entry.mark_sent()

    actual_ids = [str(entry.get_id()) for entry in actual_entries]
    cleaned_ids = DjangoAuditLogSource.clear_sent_entries(0)

    assert len(actual_ids) == num_objects
    assert len(cleaned_ids) == num_objects

    for cleaned_id in cleaned_ids:
        assert cleaned_id in actual_ids

    cleaned_ids = DjangoAuditLogSource.clear_sent_entries(0)
    assert len(cleaned_ids) == 0


def test_optional_django_audit_log():
    with patch.dict(
        "sys.modules", {"resilient_logger.sources.django_audit_log_source": None}
    ):
        import resilient_logger.sources as sources

        importlib.reload(sources)

        with pytest.raises(ImportError):
            sources.DjangoAuditLogSource()



# Generated by Django 4.2.25 on 2025-10-29 06:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resilient_logger", "0002_alter_verbose_name"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="resilientlogentry",
            options={
                "ordering": ["-created_at", "-id"],
                "verbose_name": "resilient log entry",
                "verbose_name_plural": "resilient log entries",
            },
        ),
        migrations.AlterField(
            model_name="resilientlogentry",
            name="created_at",
            field=models.DateTimeField(
                auto_now_add=True, db_index=True, verbose_name="created at"
            ),
        ),
    ]



import typing

from resilient_logger.utils import unavailable_class

from .abstract_log_source import AbstractLogSource as AbstractLogSource
from .resilient_log_source import ResilientLogSource as ResilientLogSource

if typing.TYPE_CHECKING:
    from .django_audit_log_source import DjangoAuditLogSource as DjangoAuditLogSource
else:
    try:
        from .django_audit_log_source import (
            DjangoAuditLogSource as DjangoAuditLogSource,
        )
    except ImportError:
        DjangoAuditLogSource = unavailable_class(
            "DjangoAuditLogSource", ["django-auditlog"]
        )



from collections.abc import Iterator
from datetime import timedelta

from auditlog.models import LogEntry
from django.contrib.auth.models import AbstractUser
from django.db import transaction
from django.db.models import Q
from django.utils import timezone

from resilient_logger.sources import AbstractLogSource
from resilient_logger.sources.abstract_log_source import AuditLogDocument
from resilient_logger.utils import get_resilient_logger_config


class DjangoAuditLogSource(AbstractLogSource):
    def __init__(self, log: LogEntry):
        self.log = log

    def get_id(self) -> str | int:
        return self.log.id

    def get_document(self) -> AuditLogDocument:
        config = get_resilient_logger_config()
        actor: AbstractUser | None = self.log.actor

        # Looks up the action tuple [int, str] and uses name of it
        action = LogEntry.Action.choices[self.log.action][1]
        additional_data = (self.log.additional_data or {}).copy()

        # Remove is_sent variable from additional_data, it's only for local tracking
        additional_data.pop("is_sent", None)

        extra = {
            **additional_data,
            "changes": self.log.changes,
            "source_pk": self.get_id(),
        }

        return {
            "@timestamp": self.log.timestamp,
            "audit_event": {
                "actor": self._parse_actor(actor),
                "date_time": self.log.timestamp,
                "operation": str(action).upper(),
                "origin": config["origin"],
                "target": {
                    "value": self.log.object_repr,
                },
                "environment": config["environment"],
                "message": self.log.changes_str,
                "extra": extra,
            },
        }

    def is_sent(self) -> bool:
        if self.log.additional_data is None:
            return False

        if isinstance(self.log.additional_data, dict):
            return self.log.additional_data.get("is_sent", False)

        return False

    def mark_sent(self) -> None:
        if self.log.additional_data is None:
            self.log.additional_data = {}

        self.log.additional_data["is_sent"] = True
        self.log.save(update_fields=["additional_data"])

    @classmethod
    @transaction.atomic
    def get_unsent_entries(cls, chunk_size: int) -> Iterator["DjangoAuditLogSource"]:
        entries = (
            LogEntry.objects.select_related("actor")
            .filter(
                (
                    ~Q(additional_data__has_key="is_sent")  # support old entries
                    | Q(additional_data__is_sent=False)
                ),
            )
            .order_by("timestamp")
            .iterator(chunk_size=chunk_size)
        )

        for entry in entries:
            yield cls(entry)

    @classmethod
    @transaction.atomic
    def clear_sent_entries(cls, days_to_keep: int = 30) -> list[str]:
        entries = LogEntry.objects.filter(
            ~Q(additional_data__has_key="is_sent")  # support old entries
            | Q(additional_data__is_sent=True),
            timestamp__lte=(timezone.now() - timedelta(days=days_to_keep)),
        ).select_for_update()

        deleted_ids = list(entries.values_list("id", flat=True))
        entries.delete()

        return [str(deleted_id) for deleted_id in deleted_ids]

    @staticmethod
    def _parse_actor(raw_actor: AbstractUser | None) -> dict:
        if raw_actor:
            return {"name": raw_actor.get_full_name(), "email": raw_actor.email}

        return {"name": None, "email": None}



from abc import ABC, abstractmethod
from collections.abc import Iterator
from datetime import datetime
from typing import TypedDict, TypeVar


class AuditLogEvent(TypedDict):
    actor: dict
    date_time: datetime
    operation: str
    origin: str
    target: dict
    environment: str
    message: str
    level: int | None
    extra: dict | None


AuditLogDocument = TypedDict(
    "AuditLogDocument",
    {
        "@timestamp": str,
        "audit_event": AuditLogEvent,
    },
)

TAbstractLogSource = TypeVar("TAbstractLogSource", bound="AbstractLogSource")


class AbstractLogSource(ABC):
    """
    Abstract base class (interface) that defines the method signatures.
    This is required because Django will not work if we import something
    that relies on Models too early.
    """

    @abstractmethod
    def get_id(self) -> str | int:
        raise NotImplementedError()

    @abstractmethod
    def get_document(self) -> AuditLogDocument:
        raise NotImplementedError()

    @abstractmethod
    def is_sent(self) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def mark_sent(self) -> None:
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def get_unsent_entries(
        cls: type[TAbstractLogSource], chunk_size: int
    ) -> Iterator[TAbstractLogSource]:
        """
        Queries and returns iterator for unsent log entries.
        """
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def clear_sent_entries(cls, days_to_keep: int = 30) -> list[str]:
        """
        Clears the old entries that are older than days_to_keep days
        and returns the list of the cleared object ids.
        """
        raise NotImplementedError()



from abc import ABC, abstractmethod

from resilient_logger.sources import AbstractLogSource


class AbstractLogTarget(ABC):
    def __init__(self, required: bool = True):
        """
        Base class for logging targets.
        Required means if we can proceed even if the target fails.
        """
        self.required = required

    def is_required(self) -> bool:
        return self.required

    @abstractmethod
    def submit(self, entry: AbstractLogSource) -> bool:
        raise NotImplementedError()



from unittest.mock import Mock

import pytest
from django.contrib.admin import AdminSite
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AbstractUser

from resilient_logger.admin import ResilientLogEntryAdmin
from resilient_logger.models import ResilientLogEntry


@pytest.fixture(name="superuser", autouse=True)
def superuser() -> AbstractUser:
    User = get_user_model()  # noqa: N806
    return User.objects.create_superuser("admin", "admin@example.com", "admin")


@pytest.mark.django_db
def test_resilient_logger_admin_message_prettified(superuser: AbstractUser):
    request = Mock(user=superuser)
    model_admin = ResilientLogEntryAdmin(ResilientLogEntry, AdminSite())
    assert list(model_admin.get_fields(request)) == [
        "id",
        "is_sent",
        "level",
        "created_at",
        "message_prettified",
        "context_prettified",
    ]


@pytest.mark.django_db
def test_resilient_logger_admin_permissions(superuser: AbstractUser):
    request = Mock(user=superuser)
    log_entry = ResilientLogEntry.objects.create(message={})
    model_admin = ResilientLogEntryAdmin(ResilientLogEntry, AdminSite())
    # The user should have permission to view
    # but not modify or delete resilient logs
    assert model_admin.has_view_permission(request, log_entry)
    assert not model_admin.has_add_permission(request)
    assert not model_admin.has_change_permission(request, log_entry)
    assert not model_admin.has_delete_permission(request, log_entry)



from django.db import models
from django.utils.translation import gettext_lazy as _


class ResilientLogEntry(models.Model):
    is_sent = models.BooleanField(default=False, verbose_name=_("is sent"))
    level = models.IntegerField(verbose_name=_("level"), default=0)
    message = models.JSONField(verbose_name=_("message"))
    context = models.JSONField(verbose_name=_("context"), null=True)
    created_at = models.DateTimeField(
        auto_now_add=True, verbose_name=_("created at"), db_index=True
    )

    class Meta:
        ordering = ["-created_at", "-id"]
        verbose_name = _("resilient log entry")
        verbose_name_plural = _("resilient log entries")



import hashlib
import json
import logging
from collections.abc import Callable, Sequence
from functools import cache
from importlib import import_module
from typing import Any, TypedDict, TypeVar, cast

from django.conf import settings
from django.core.serializers.json import DjangoJSONEncoder

from resilient_logger.errors.missing_context_error import MissingContextError


class ResilientLoggerConfig(TypedDict):
    origin: str
    environment: str
    batch_limit: int
    chunk_size: int
    submit_unsent_entries: bool
    clear_sent_entries: bool
    sources: list[dict[str, Any]]
    targets: list[dict[str, Any]]


def _non_empty_string(input: str) -> bool:
    return bool(input.strip())


def _non_empty_list(input: list) -> bool:
    return len(input) > 0


_required_fields: tuple[tuple[str, type[Any], Callable[[Any], bool] | None], ...] = (
    ("environment", str, _non_empty_string),
    ("origin", str, _non_empty_string),
    ("sources", list, _non_empty_list),
    ("targets", list, _non_empty_list),
)

_default_config: ResilientLoggerConfig = {
    "batch_limit": 5000,
    "chunk_size": 500,
    "clear_sent_entries": False,
    "submit_unsent_entries": False,
}

BUILTIN_LOG_RECORD_ATTRS = {
    "args",
    "asctime",
    "created",
    "exc_info",
    "exc_text",
    "filename",
    "funcName",
    "levelname",
    "levelno",
    "lineno",
    "module",
    "msecs",
    "message",
    "msg",
    "name",
    "pathname",
    "process",
    "processName",
    "relativeCreated",
    "stack_info",
    "taskName",
    "thread",
    "threadName",
}

TClass = TypeVar("TClass")


def dynamic_class(type: type[TClass], class_path: str) -> type[TClass]:
    """
    Loads dynamically class of given type from class_path
    and ensures it's sub-class of given input type.
    """
    parts = class_path.split(".")
    class_name = parts.pop()
    module_name = ".".join(parts)
    module = import_module(module_name)
    cls = getattr(module, class_name)

    if not issubclass(cls, type):
        raise TypeError(f"Class '{class_path}' is not sub-class of the {type}.")

    return cls


def get_log_record_extra(record: logging.LogRecord):
    """Returns `extra` passed to the logger."""
    return {
        name: record.__dict__[name]
        for name in record.__dict__
        if name not in BUILTIN_LOG_RECORD_ATTRS
    }


def assert_required_extras(extra: dict[str, Any], required_fields: list[str]) -> None:
    missing_fields = [field for field in required_fields if extra.get(field) is None]
    if missing_fields:
        raise MissingContextError(missing_fields)


@cache
def get_resilient_logger_config() -> ResilientLoggerConfig:
    raw_config: ResilientLoggerConfig | None = getattr(
        settings, "RESILIENT_LOGGER", None
    )

    if raw_config is None:
        raise RuntimeError("RESILIENT_LOGGER setting is missing")

    if not isinstance(raw_config, dict):
        raise RuntimeError("RESILIENT_LOGGER is not a proper dictionary")

    config = {**_default_config, **raw_config}

    for key, expected_type, validator in _required_fields:
        if key not in config:
            raise RuntimeError(f"RESILIENT_LOGGER is missing required key: '{key}'")

        value = config[key]

        if not isinstance(value, expected_type):
            actual_type = type(value).__name__
            raise RuntimeError(
                f"RESILIENT_LOGGER['{key}'] must be {expected_type.__name__}, "
                f"got {actual_type}"
            )

        if validator and not validator(value):
            raise RuntimeError(f"RESILIENT_LOGGER['{key}'] failed validation")

    return cast(ResilientLoggerConfig, config)


def content_hash(contents: dict[str, Any]) -> str:
    json_repr = json.dumps(contents, sort_keys=True, cls=DjangoJSONEncoder)
    return hashlib.sha256(json_repr.encode()).hexdigest()


def unavailable_class(name: str, dependencies: Sequence[str]):
    """
    Creates a placeholder class that raises ImportError on instantiation.

    Parameters:
        name (str): Name of the class (for nicer repr).
        dependency (str): The missing dependency to mention in the error.
    """
    deps = ", ".join(f"'{d}'" for d in dependencies)

    class _UnavailableClass:
        def __init__(self, *args, **kwargs):
            raise ImportError(f"{name} requires the optional dependencies: {deps}. ")

        def __repr__(self):
            return f"<Unavailable class {name} (missing dependencies: {deps})>"

    _UnavailableClass.__name__ = name
    return _UnavailableClass


def value_as_dict(value: str | dict) -> dict:
    if isinstance(value, str):
        return {"value": value}

    if isinstance(value, dict):
        return value

    value_type = type(value).__name__

    raise TypeError(
        f"Invalid value_as_dict input. Expected 'str | dict', got '{value_type}'"
    )



import logging
from collections.abc import Iterator
from typing import TypeVar, cast

from django.db import transaction

from resilient_logger.sources import AbstractLogSource
from resilient_logger.targets import AbstractLogTarget
from resilient_logger.utils import dynamic_class, get_resilient_logger_config

logger = logging.getLogger(__name__)

TResilientLogger = TypeVar("TResilientLogger", bound="ResilientLogger")


class ResilientLogger:
    def __init__(
        self,
        batch_limit: int,
        chunk_size: int,
        log_sources: list[type[AbstractLogSource]],
        log_targets: list[AbstractLogTarget],
    ) -> None:
        self._batch_limit = batch_limit
        self._chunk_size = chunk_size
        self._log_sources = log_sources
        self._log_targets = log_targets

    @classmethod
    def create(cls: type[TResilientLogger]) -> TResilientLogger:
        settings = get_resilient_logger_config()
        batch_limit = settings.get("batch_limit", 5000)
        chunk_size = settings.get("chunk_size", 500)
        sources = settings.get("sources", []).copy()
        targets = settings.get("targets", []).copy()

        list_sources: list[type[AbstractLogSource]] = []
        list_targets: list[AbstractLogTarget] = []

        for source in sources:
            source_args = source.copy()
            source_class_name = source_args.pop("class", None)
            source_class = dynamic_class(
                cast(type[AbstractLogSource], AbstractLogSource), source_class_name
            )
            list_sources.append(source_class)

        for target in targets:
            target_args = target.copy()
            target_class_name = target_args.pop("class", None)
            target_class = dynamic_class(
                cast(type[AbstractLogTarget], AbstractLogTarget), target_class_name
            )
            list_targets.append(target_class(**target_args))

        return cls(
            batch_limit=batch_limit,
            chunk_size=chunk_size,
            log_sources=list_sources,
            log_targets=list_targets,
        )

    @transaction.atomic
    def submit_unsent_entries(self) -> dict[str, bool]:
        results: dict[str, bool] = {}

        for count, entry in enumerate(self._get_unsent_entries()):
            if count >= self._batch_limit:
                logger.info(f"Job limit of {self._batch_limit} logs reached.")
                break

            entry_id = str(entry.get_id())
            result = self._submit(entry)
            results[entry_id] = result

            if result:
                entry.mark_sent()

        return results

    @transaction.atomic
    def clear_sent_entries(self, days_to_keep: int = 30) -> list[str]:
        """
        Clears all the old entries that are older than days_to_keep days
        and returns the list of the cleared object ids.
        """
        deleted_ids: list[str] = []

        for log_source in self._log_sources:
            deleted_ids += log_source.clear_sent_entries(days_to_keep)

        return deleted_ids

    def _submit(self, source: AbstractLogSource) -> bool:
        for log_target in self._log_targets:
            submitted = False

            try:
                submitted = log_target.submit(source)
            except Exception:
                logger.exception("Log target threw while submitting")

            if not submitted and log_target.is_required():
                return False

        return True

    def _get_unsent_entries(self) -> Iterator[AbstractLogSource]:
        """
        Queries and returns iterator for all unsent log entries.
        """
        for log_source in self._log_sources:
            yield from log_source.get_unsent_entries(self._chunk_size)



class MissingContextError(Exception):
    def __init__(self, missing_fields: list[str]):
        self.missing_fields = missing_fields

    def __str__(self):
        entries = ",".join(self.missing_fields)
        return f"Log entry is missing required context entries: [{entries}]"



VALID_CONFIG_ALL_FIELDS = {
    "sources": [
        {
            "class": "resilient_logger.sources.ResilientLogSource",
        }
    ],
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "dev",
    "origin": "test",
}

VALID_CONFIG_MISSING_OPTIONAL = {
    "sources": [
        {
            "class": "resilient_logger.sources.ResilientLogSource",
        }
    ],
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "environment": "dev",
    "origin": "test",
}

INVALID_CONFIG_MISSING_TARGETS = {
    "sources": [
        {
            "class": "resilient_logger.sources.ResilientLogSource",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "dev",
    "origin": "test",
}

INVALID_CONFIG_EMPTY_TARGETS = {
    "sources": [
        {
            "class": "resilient_logger.sources.ResilientLogSource",
        }
    ],
    "targets": [],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "dev",
    "origin": "test",
}

INVALID_CONFIG_MISSING_SOURCES = {
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "dev",
    "origin": "test",
}

INVALID_CONFIG_EMPTY_SOURCES = {
    "sources": [],
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "dev",
    "origin": "test",
}

INVALID_CONFIG_MISSING_ENVIRONMENT = {
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "origin": "test",
}

INVALID_CONFIG_EMPTY_ENVIRONMENT = {
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "",
    "origin": "test",
}

INVALID_CONFIG_MISSING_ORIGIN = {
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "dev",
}

INVALID_CONFIG_EMPTY_ORIGIN = {
    "targets": [
        {
            "class": "resilient_logger.targets.ProxyLogTarget",
            "name": "proxy-target",
        }
    ],
    "batch_limit": 5000,
    "chunk_size": 500,
    "submit_unsent_entries": True,
    "clear_sent_entries": True,
    "environment": "dev",
    "origin": "",
}



# Generated by Django 4.2.25 on 2025-10-29 05:35

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resilient_logger", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="resilientlogentry",
            options={
                "verbose_name": "resilient log entry",
                "verbose_name_plural": "resilient log entries",
            },
        ),
    ]



import logging

from django.core.management.base import BaseCommand

from resilient_logger.resilient_logger import ResilientLogger
from resilient_logger.utils import get_resilient_logger_config

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Send django-resilient-logger entries to centralized log center"

    def __init__(self):
        super().__init__()
        settings = get_resilient_logger_config()
        self.should_submit = settings["submit_unsent_entries"] or False
        self.resilient_logger = ResilientLogger.create()

    def handle(self, *args, **options):
        if not self.should_submit:
            logger.info("submit_unsent_entries is disabled in config")
            return

        logger.info("Begin submit_unsent_entries job.")
        result = self.resilient_logger.submit_unsent_entries()
        logger.info("Finished submit_unsent_entries done.", extra={"result": result})



from auditlog.registry import auditlog
from django.db import models
from django.utils.translation import gettext_lazy as _


class DummyModel(models.Model):
    message = models.JSONField(verbose_name=_("message"))

    class Meta:
        verbose_name = _("dummy model")
        verbose_name_plural = _("dummy models")


auditlog.register(DummyModel)
